#include <avr/io.h>          //nzvy registr
#include <stddef.h>
#include <stdio.h>
#include <inttypes.h>
#include <stdlib.h>
#include <util/atomic.h>
#include <math.h>

#include "posreg.h"


//******************************************************************************
//******************************************************************************
TPosregS PosregS;
//******************************************************************************
void posregInit(void);
//******************************************************************************
u8t
posregAction(u32t pos, double veloci, double accel, TPosregS* sp);
//******************************************************************************
void
posregStop(TPosregS* sp);
//******************************************************************************
static u32t posregAceleration(u32t s, double a);
//******************************************************************************
static inline void
posregStep(TPosregS* sp);
//******************************************************************************
static void
posregSetOCR1A(u32t time);
//******************************************************************************
//only 4,8,
const u8t
StepSeq[]={
//0b0001,0b0010,0b0100,0b1000
//0b0011,0b0110,0b1100,0b1001
0b0011,0b0010,0b0110,0b0100,0b1100,0b1000,0b1001,0b0001,
};
//******************************************************************************
//               Peruen pi komparaci s OCR1A u tae 1
//******************************************************************************
ISR(TIMER1_COMPA_vect){

	if(PosregS.Status.ActionE == POSREG_END){
		TIMSK &= ~_BV(OCIE1A);
		PosregS.Status.ActionE = POSREG_STOP;
		return;
	}

	posregStep(&PosregS);

	if( PosregS.Calc.DifPos == 0 ){
		PosregS.Calc.NCt = posregAceleration(1, PosregS.Actual.Accel)/2;
		posregSetOCR1A(PosregS.Calc.NCt);
		PosregS.Actual.runTime += PosregS.Calc.NCt;
		PosregS.Status.ActionE = POSREG_END;
		return;
	}

	if( PosregS.Calc.RelPos >= PosregS.Calc.RampUpLen ){
		PosregS.Status.ActionE = POSREG_LINEAR;
	}

	if( PosregS.Calc.DifPos <= PosregS.Calc.RampDownLen ){
		PosregS.Status.ActionE = POSREG_DECEL;
	}

	switch(PosregS.Status.ActionE){
		//*****************
		case POSREG_ACCELER:
			PosregS.Calc.NCt = posregAceleration(PosregS.Calc.RelPos, PosregS.Actual.Accel);
			break;
		//*****************
		case POSREG_LINEAR:
			PosregS.Calc.NCt = PosregS.Calc.NCtMin;
			break;
		//*****************
		case POSREG_DECEL:
			PosregS.Calc.NCt = posregAceleration(PosregS.Calc.DifPos, PosregS.Actual.Accel);
			break;
		default:
			break;
	}
	PosregS.Actual.runTime += PosregS.Calc.NCt;
	posregSetOCR1A(PosregS.Calc.NCt);
}
//******************************************************************************
void
posregSetOCR1A(u32t time){
	u8t tempU8=1;

	while( (time > U16TOP)&&(tempU8 < (sizeof(PrescDif)/sizeof(u8t))) ){
		time >>= PrescDif[tempU8++];
	}

	if(time > U16TOP){
		time = U16TOP;
	}

	TCCR1B = (TCCR1B&(~0b111))|tempU8;
	OCR1A = time;
}
//******************************************************************************
static inline void
posregStep(TPosregS* sp){
	if(sp->Calc.Direct == POSREG_RIGHT){
		sp->Actual.StepPhase ++;
		sp->Actual.Pos ++;
	}else{
		sp->Actual.StepPhase --;
		sp->Actual.Pos --;
	}
	sp->Actual.StepPhase &= ((sizeof(StepSeq)/sizeof(u8t))-1);
	sp->Calc.RelPos++;
	sp->Calc.DifPos--;
	POSREG_OUTPORT = StepSeq[sp->Actual.StepPhase];

}
//******************************************************************************
//******************************************************************************
u32t posregAceleration(u32t s, double a){
	return POSREG_CTC_FREQ/sqrt(2*s*a);
}
//******************************************************************************
//Vypocita potrebne hodnoty pro Calc
//POSREG_CTC_FREQ*msDelay<S32TOP
void
posregCalc(s32t msDelay, TPosregS* sp){
	u32t tempU32;
//	Probiha vypocet
	sp->Status.ActionE = POSREG_CALC;

//	Zkopiruj a omez rychlost
	sp->Actual.Veloci = sp->Action.Veloci;
	if(sp->Actual.Veloci > POSREG_MAXVELOCI)sp->Actual.Veloci = POSREG_MAXVELOCI;
	if(sp->Actual.Veloci < POSREG_MINVELOCI)sp->Actual.Veloci = POSREG_MINVELOCI;

//	Zkopiruj a omez akceleraci
	sp->Actual.Accel = sp->Action.Accel;
	if(sp->Actual.Accel > POSREG_MAXACCEL)sp->Actual.Accel = POSREG_MAXACCEL;
	if(sp->Actual.Accel < POSREG_MINACCEL)sp->Actual.Accel = POSREG_MINACCEL;

//	spocitej nejkraci cas mezi kroky
	sp->Calc.NCtMin = POSREG_CTC_FREQ / sp->Actual.Veloci;

//	vynuluj
	sp->Calc.RelPos = 0;

//	urci smer a diferenc polohy
	if(sp->Action.SetPos > sp->Actual.Pos){
		sp->Calc.DifPos = sp->Action.SetPos - sp->Actual.Pos;
		sp->Calc.Direct = POSREG_RIGHT;
	}else{
		sp->Calc.DifPos = sp->Actual.Pos - sp->Action.SetPos;
		sp->Calc.Direct = POSREG_LEFT;
	}
//
	if( sp->Calc.DifPos == 0){
		sp->Status.ActionE = POSREG_STOP;
		return;
	}

//	vypocitej delku rampy
	tempU32 = ((sp->Actual.Veloci*sp->Actual.Veloci) / sp->Actual.Accel);
	if(tempU32 >= sp->Calc.DifPos){
		//	pokud je rampa delsi nez diferenc polohy
		sp->Calc.RampUpLen = sp->Calc.DifPos/2;
		sp->Calc.RampDownLen = sp->Calc.DifPos - sp->Calc.RampUpLen;
	}else{
		//
		sp->Calc.RampUpLen = tempU32/2;
		sp->Calc.RampDownLen = sp->Calc.RampUpLen;
	}

	tempU32 = posregAceleration(1, PosregS.Actual.Accel)/2;
	//kontrola	preteceni a podteceni
	//***#warning
	tempU32+= (POSREG_CTC_FREQ*msDelay)/1000;

	posregSetOCR1A(tempU32);

	sp->Actual.runTime=0;
	sp->Status.ActionE = POSREG_ACCELER;

	TIFR |= _BV(OCF1A);
	TCNT1 = 0;
	TIMSK|=_BV(OCIE1A);
}
//******************************************************************************
u8t
posregAction(u32t pos, double veloci, double accel, TPosregS* sp){
	
	if((PosregS.Status.ActionE == POSREG_STOP)&&(PosregS.Status.OperateE == POSREG_STAT_NORMAL)){
		sp->Action.SetPos = pos;
		sp->Action.Veloci = veloci;
		sp->Action.Accel = accel;
		posregCalc(0,sp);
		return 1;
	}
	return 0;
}
//******************************************************************************
u8t
posregStartAction(TPosregS* sp){

	if((PosregS.Status.ActionE == POSREG_STOP)&&(PosregS.Status.OperateE == POSREG_STAT_NORMAL)){
		posregCalc(0,sp);
		return 1;
	}
	return 0;
}
//******************************************************************************
void
posregStop(TPosregS* sp){
		TIMSK &= ~_BV(OCIE1A);
		sp->Actual.StepPhase = 0;
		POSREG_OUTPORT = StepSeq[sp->Actual.StepPhase];
		sp->Status.ActionE = POSREG_STOP;
		sp->Status.HomingE = POSREG_HOMING_NOHOME;
}
//******************************************************************************
void
posregSetHomePos(u32t pos,TPosregS* sp){
 sp->Actual.Pos = pos;
 sp->Status.HomingE = POSREG_HOMING_OK;
 sp->Status.OperateE = POSREG_STAT_NORMAL;

}
//******************************************************************************
void posreg(TPosregS* sp){

	if(sp->Status.OperateE == POSREG_STAT_HOMING){
		switch(sp->Status.HomingE){
		//**********************************
			case POSREG_HOMING_NOHOME:
				TIMSK &= ~_BV(OCIE1A);
				sp->Status.ActionE = POSREG_STOP;
				sp->Actual.Pos=POSREG_HOMING_LEN;
				sp->Action.SetPos = 0;
				sp->Action.Veloci = POSREG_HOMING_VELOCI1;
				sp->Action.Accel = POSREG_HOMING_ACCEL1;
				posregCalc(POSREG_HOMING_DELAY1,sp);
				sp->Status.HomingE++;
				break;
		//**********************************
			case POSREG_HOMING_GOHOME1:
				//pokud nalezne home = 1
				if(POSREG_HOMING_INPUT){
					TIMSK &= ~_BV(OCIE1A);
					sp->Status.ActionE = POSREG_STOP;
					sp->Actual.Pos=0;
					sp->Action.SetPos = POSREG_HOMING_HYSTERZ;
					sp->Action.Veloci = POSREG_HOMING_VELOCI2;
					sp->Action.Accel = POSREG_HOMING_ACCEL2;
					posregCalc(POSREG_HOMING_DELAY1,sp);
					sp->Status.HomingE++;
					break;	
				}
				//pokud nenalezne home
				if(PosregS.Status.ActionE == POSREG_STOP){
					sp->Status.OperateE = POSREG_HOMING_ERR;
				}
				break;
		//**********************************
			case POSREG_HOMING_BACK:
				if((PosregS.Status.ActionE == POSREG_STOP)|(!POSREG_HOMING_INPUT)){
					if(POSREG_HOMING_INPUT){
						//nesjel z koncaku
						sp->Status.HomingE = POSREG_HOMING_ERR;
						break;
					}
					TIMSK &= ~_BV(OCIE1A);
					sp->Status.ActionE = POSREG_STOP;
					sp->Actual.Pos = POSREG_HOMING_HYSTERZ*2;
					sp->Action.SetPos = 0;
					sp->Action.Veloci = POSREG_HOMING_VELOCI2;
					sp->Action.Accel = POSREG_HOMING_ACCEL2;
					posregCalc(POSREG_HOMING_DELAY2,sp);
					sp->Status.HomingE++;
				}
				break;
 		//**********************************
			case POSREG_HOMING_GOHOME2:
				//pokud nalezne home
				if(POSREG_HOMING_INPUT){
					TIMSK &= ~_BV(OCIE1A);
					sp->Status.ActionE = POSREG_STOP;
					sp->Actual.Pos=0;
					sp->Action.SetPos = POSREG_HOMING_HYSTERZ;
					sp->Action.Veloci = POSREG_HOMING_VELOCI2;
					sp->Action.Accel = POSREG_HOMING_ACCEL2;
					posregCalc(POSREG_HOMING_DELAY2,sp);
					sp->Status.HomingE++;
					break;	
				}
				//pokud nenalezne home
				if(PosregS.Status.ActionE == POSREG_STOP){
					sp->Status.HomingE = POSREG_HOMING_ERR;
				}
				break;
		//**********************************
			case POSREG_HOMING_OK:
				if(PosregS.Status.ActionE == POSREG_STOP){
					sp->Actual.Pos=0;
					sp->Status.OperateE = POSREG_STAT_NORMAL;
				}
				break;
		//**********************************
			case POSREG_HOMING_ERR:
				TIMSK &= ~_BV(OCIE1A);
				sp->Status.ActionE = POSREG_STOP;
				break;
		//**********************************
			default:
			sp->Status.OperateE = POSREG_STAT_NORMAL;
			sp->Status.HomingE = POSREG_HOMING_ERR;
		}
	}else{
		if(POSREG_HOMING_INPUT){
			TIMSK &= ~_BV(OCIE1A);
			sp->Status.ActionE = POSREG_STOP;
			sp->Status.HomingE = POSREG_HOMING_ERR;
		}
	
	}
}
//******************************************************************************
void
posregInit(void){

POSREG_OUTDDR;
POSREG_OUTPORT = StepSeq[0];
	//******************************************************************************
	//               Nastaven tae 1
	//******************************************************************************
	TCCR1A=((0<<COM1A1)|(0<<COM1A0)|(0<<COM1B1)|(0<<COM1B0)|(0<<WGM11)|(0<<WGM10));
	TCNT1=0;
	OCR1A=0xffff;
	OCR1B=0;
	ICR1=0;
	TCCR1B=((0<<ICNC1)|(0<<ICES1)|(0<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10));
}
