#ifndef POSREG_H
#define POSREG_H

 #ifndef TYPEDEFINE
 #define TYPEDEFINE
	typedef uint8_t		u8t;
	typedef uint16_t	u16t;
	typedef uint32_t	u32t;

	typedef int8_t		s8t;
	typedef int16_t		s16t;
	typedef int32_t		s32t;

	#define U8TOP ((u8t)(~0))
	#define U16TOP ((u16t)(~0))
	#define U32TOP ((u32t)(~0))

	#define S8TOP ((s8t)(((u8t)U8TOP) >>1))
	#define S8BUTT (S8TOP+1)

	#define S16TOP ((s16t)(((u16t)U16TOP) >>1))
	#define S16BUTT (S16TOP+1)

	#define S32TOP ((s32t)(((u32t)U32TOP) >>1))
	#define S32BUTT (S32TOP+1)

 #endif

#include "../hw.h"

#ifndef POSREG_HW_H
#define POSREG_HW_H
#warning HW config is default

	//vystupni port pro motor
	#define POSREG_OUTPORT PORTA //porta
	//vystupy, zapne branu pro motor a sleep
	#define POSREG_OUTDDR {DDRA|=0xf;DDRA|=_BV(PA0);} //ddre
	//cidlo home 1-home
	#define POSREG_HOMING_INPUT (PINA&_BV(PA0))
	//
	#define POSREG_MOTOR_PORT_HI {PORTA|=_BV(PA1);}
	#define POSREG_MOTOR_PORT_LO {PORTA&=~_BV(PA1);}


	//Frekvence citace
	#define POSREG_CTC_FREQ 16000000

	#define POSREG_MINVELOCI 0.01
	// !!!Nepretizit procesor!!!
	#define POSREG_MAXVELOCI 1000.0

	#define POSREG_MINDECEL 0.01
	#define POSREG_MAXDECEL 10000

	#define POSREG_MINACCEL 0.01
	#define POSREG_MAXACCEL 10000


	//Pri pokynu go home se motor vyda smerem k cidlu po max vzdalenost POSREG_HOMING_LEN
	//Pri nalezeni cidla se motor vyda zpet o polohu POSREG_HOMING_HYSTERZ
	//pote se vyda opet k cidlu maximalne o polohu 2xPOSREG_HOMING_HYSTERZ ale jiz mensi rychlosti.
	//Po dosazeni cidla motor popojede o POSREG_HOMING_HYSTERZ dopredu a tam nastavi nulu
	//maximalni delka hledani cidla
	#define POSREG_HOMING_LEN 10000
	//rychlost 1. hrubeho vyhledavani
	#define POSREG_HOMING_VELOCI1 30
	#define POSREG_HOMING_ACCEL1 30
	//prodleva pred roztocenim v ms
	#define POSREG_HOMING_DELAY1 1000
	//rychlost 2. jemneho vyhledavani
	#define POSREG_HOMING_VELOCI2 5

	#define POSREG_HOMING_ACCEL2 1
	//prodleva pred roztocenim v ms
	#define POSREG_HOMING_DELAY2 1000
	//hysterze pro 2. navrat na cidlo a zpet do polohy 0
	#define POSREG_HOMING_HYSTERZ 15

#endif
//******************************************************************************
//mocniny podil mezi prescaleri citace
static const u8t PrescDif[5]={0, 3, 3, 2, 2};

//******************************************************************************
//******************************************************************************
typedef enum {POSREG_RIGHT, POSREG_LEFT}TPosregDirectionE;
typedef enum {POSREG_STOP, POSREG_CALC, POSREG_ACCELER, POSREG_LINEAR, POSREG_DECEL, POSREG_END }TPosregActionE;
typedef enum {	POSREG_HOMING_NOHOME,
				POSREG_HOMING_GOHOME1,
				POSREG_HOMING_BACK,
				POSREG_HOMING_GOHOME2,
				POSREG_HOMING_OK,
				POSREG_HOMING_ERR
			}TPosregHomingE;
typedef enum {POSREG_STAT_NORMAL,POSREG_STAT_HOMING}TPosregOperateE;
typedef enum {POSREG_PWR_MOTORLO,POSREG_PWR_MOTORHI}TPosregPwrModeE;
//******************************************************************************
//******************************************************************************
typedef struct{
	u32t SetPos;
	double Veloci;
	double Accel;
}TPosregActionS;
//******************************************************************************
typedef struct{
	u32t Pos;
	double Veloci;
	double Accel;
	double runTime;
	u8t StepPhase;
}TPosregActualS;
//******************************************************************************
//RelPos - pred kazdym polohovanim se vynuluje a inkrementuje se vzdy do plusu
//NCtMin - perioda pri maxVeloci
typedef struct{
	u32t NCt;
	u32t NCtMin;
	u32t RelPos;
	u32t DifPos;
	u32t RampUpLen;
	u32t RampDownLen;
	TPosregDirectionE Direct;
}TPosregCalcS;
//******************************************************************************
//******************************************************************************
typedef struct{
	TPosregActionE ActionE;
	TPosregHomingE HomingE;
	TPosregPwrModeE PwrModeE;
	TPosregOperateE OperateE;

}TPosregStatusS;
//******************************************************************************
//******************************************************************************
typedef volatile struct{
	TPosregActionS Action;
	TPosregActualS Actual;
	TPosregCalcS Calc;
	TPosregStatusS Status;

}TPosregS;
//******************************************************************************
//******************************************************************************
extern TPosregS PosregS;

//******************************************************************************
//******************************************************************************
void posregInit(void);
//******************************************************************************
void posreg(TPosregS* sp);
//******************************************************************************
u8t posregAction(u32t pos, double veloci, double accel, TPosregS* sp);
//******************************************************************************
u8t posregStartAction(TPosregS* sp);
//******************************************************************************
void posregStop(TPosregS* sp);
//******************************************************************************
void posregSetHomePos(u32t pos,TPosregS* sp);
//******************************************************************************
#endif
