#include <avr/io.h>          //nzvy registr
#include <stddef.h>
#include <stdio.h>
#include <inttypes.h>
#include <stdlib.h>
#include <util/atomic.h>
#include <math.h>
#include <avr/pgmspace.h>

#include "../posreg/posreg.h"

#include "usartservo.h"


//******************************************************************************
//Peruen od pjmu znaku
ISR(USART_RXC_vect){
	u8t udr_in;
	u8t hw_status;

	hw_status = UCSRA;
	udr_in = UDR;

	UssBusS.Diag.rxLedI = USS_RXLED;

	//chybne prijato
	if ( hw_status & ((1<<FE)|(1<<DOR)|(1<<PE)) ){  
		UssBusS.Recv.sw = USS_RX_START;
		UssBusS.Diag.errLedI = USS_ERRLED;
		return;
	}

	switch(UssBusS.Recv.sw){
		//prijem zakazan
		case USS_RX_DIS:
			UssBusS.Recv.sw = USS_RX_DIS;
			UCSRB &= ~_BV(RXCIE);
		return;
		//cekam na prijeti '{'
		case USS_RX_START:
			if(udr_in == '{'){
				UssBusS.Recv.rx_i = 0;
				UssBusS.Data.buf[UssBusS.Recv.rx_i++] = udr_in;
				UssBusS.Recv.sw++;
			}
		break;
		//prijimam adresu
		case USS_RX_ADR:
			if(udr_in == UssBusS.Data.adr + '0'){
				UssBusS.Data.buf[UssBusS.Recv.rx_i++]=udr_in;
				UssBusS.Recv.sw++;
			}else{
				UssBusS.Recv.sw = USS_RX_START;
			}
		break;
		//prijimam data a cekam na prijeti '}'
		case USS_RX_DATA:
			if(udr_in == '}'){
				UssBusS.Data.buf[UssBusS.Recv.rx_i++]=udr_in;
				UssBusS.Data.buf[UssBusS.Recv.rx_i++]=0;
				UssBusS.Recv.sw = USS_RX_OK;
				UssBusS.Diag.rxPaketI++;
				UCSRB &= ~_BV(RXCIE);
				return;
			}
			UssBusS.Data.buf[UssBusS.Recv.rx_i++] = udr_in;
			if(UssBusS.Recv.rx_i >= (sizeof(UssBusS.Data.buf)-3)){
				UssBusS.Recv.sw = USS_RX_START;
				UssBusS.Diag.errLedI = USS_ERRLED;
				return;
			}
		break;
		//prijem dokoncen a zakazan
		case USS_RX_OK:
			UssBusS.Recv.sw = USS_RX_DIS;
			UCSRB &= ~_BV(RXCIE);
		break;
	}

}
//******************************************************************************
//Peruen od przdnho datovho registru
ISR(USART_UDRE_vect){
	
	UDR = UssBusS.Data.buf[UssBusS.Transm.tx_i++];

//	if(UssBusS.Transm.tx_i >= UssBusS.Transm.txLen){
	if(UssBusS.Data.buf[UssBusS.Transm.tx_i]== 0){
		UCSRA |= _BV(TXC);
		UDR = '}';
		UCSRB &=(~_BV(UDRIE));
		UCSRB |= _BV(TXCIE);
	}
}
//******************************************************************************
//Peruen od odesln znaku
ISR(USART_TXC_vect){

	USS_RXEN;
	UCSRB &= ~_BV(TXCIE);
	UssBusS.Transm.sw = USS_TX_OK;
	UssBusS.Recv.sw = USS_RX_START;
	UssBusS.Diag.txPaketI++;
	UssBusS.Diag.txLedI = USS_TXLED;
	UCSRB |= _BV(RXCIE);
}
//******************************************************************************
static void
rxErr(void){
	UssBusS.Diag.errLedI = USS_ERRLED;
	UssRxStart();
}
//******************************************************************************
static void
status(u8t cmd){
	sprintf_P((char *)UssBusS.Data.buf,PSTR("{%d,%d,%ld,%d%d%d%d}"),
				UssBusS.Data.adr,
				cmd,
				PosregS.Actual.Pos,
				PosregS.Status.ActionE,
				PosregS.Status.HomingE,
				PosregS.Status.PwrModeE,
				PosregS.Status.OperateE);
}

//******************************************************************************
void Uss(void){
	volatile u8t dummy;
	u32t tempU32t;
	static u8t t10msI=0;
	

	if(UssBusS.Diag.msI){
		UssBusS.Diag.msI--;
		if(++t10msI>9){
			t10msI=0;
			if(UssBusS.Diag.rxLedI){
				USS_RXLEDON;UssBusS.Diag.rxLedI--;
			}else{USS_RXLEDOFF;}

			if(UssBusS.Diag.txLedI){
				USS_TXLEDON;UssBusS.Diag.txLedI--;
			}else{USS_TXLEDOFF;}
			
			if(UssBusS.Diag.errLedI){
				USS_ERRLEDON;UssBusS.Diag.errLedI--;
			}else{USS_ERRLEDOFF;}
		}
	}

	if(UssBusS.Recv.sw != USS_RX_OK)return;

	if(sscanf_P((char *)UssBusS.Data.buf,PSTR("{%d,%d,"),&dummy, &UssBusS.Recv.cmd)!=2){
		rxErr();
		return;
	}

	switch(UssBusS.Recv.cmd){
	//**********************************
		case USS_GET_STATUS:
			////{adr,0}
			status(0);
			break;
	//**********************************
		case USS_GOTO:
			//{adr,1,pos,veloci,accel}
			if(sscanf_P((char *)UssBusS.Data.buf,PSTR("{%d,%d,%ld,%f,%f}"),&dummy,&dummy,
							&PosregS.Action.SetPos,&PosregS.Action.Veloci,&PosregS.Action.Accel) !=5){
				rxErr();
				return;
			}
			//pokud motor stoji tak ho rozjed
			if(PosregS.Status.ActionE == POSREG_STOP){
				posregStartAction(&PosregS);
				status(1);			
			}else{
				//pokud motor jede posli neuspech
				status(0);
			}

			break;
	//**********************************
		case USS_GO_HOME:
			//{adr,2}
			PosregS.Status.HomingE = POSREG_HOMING_NOHOME;
			PosregS.Status.OperateE = POSREG_STAT_HOMING;
			status(2);
			break;
	//**********************************
		case USS_SET_HOME_POS:
			//{adr,3,pos}
			if(sscanf_P((char *)UssBusS.Data.buf,PSTR("{%d,%d,%ld}"),&dummy,&dummy,&tempU32t) !=3){
				rxErr();
				return;
			}
			posregStop(&PosregS);
			posregSetHomePos(tempU32t,&PosregS);
			status(3);
			break;
	//**********************************
		case USS_TOTAL_STOP:
			//{adr,4}
			posregStop(&PosregS);
			status(4);
			break;
	//**********************************
		case USS_MOTOR_LO:
			//{adr,5}
			POSREG_MOTOR_PORT_LO;
			PosregS.Status.PwrModeE = POSREG_PWR_MOTORLO;
			status(5);
			break;
	//**********************************
		case USS_MOTOR_HI:
			//{adr,6}
			POSREG_MOTOR_PORT_HI;
			PosregS.Status.PwrModeE = POSREG_PWR_MOTORHI;
			status(6);
			break;
	//**********************************
		default :
			rxErr();
			return;
	}
	UssTxStart();
}
//******************************************************************************
void UssTxStart(void){

//	UssBusS.Transm.txLen = siz;	
	UssBusS.Transm.tx_i = 0;
	UCSRB &= ~_BV(RXCIE);
	UssBusS.Recv.sw = USS_RX_DIS;
	USS_TXEN;
	UssBusS.Transm.sw = USS_TX_DATA;
	UCSRB |= _BV(TXEN)|_BV(UDRIE);
}
//******************************************************************************
void UssRxStart(void){

	USS_RXEN;
	UCSRB &= ~_BV(TXCIE);
	UssBusS.Transm.sw = USS_TX_OK;
	UssBusS.Recv.sw = USS_RX_START;
	UCSRB |= _BV(RXCIE);
}
//******************************************************************************
void Uss1ms(void){
	if(UssBusS.Diag.msI<254)UssBusS.Diag.msI++;
}
//******************************************************************************
void
ussInit(u8t adr){
	
	UssBusS.Data.adr = adr;
	
	//******************************************************************************
	//               Nastaven sriov linky
	//******************************************************************************
	UCSRA=((0<<U2X)|(0<<MPCM));
	UCSRB=((1<<RXCIE)|(0<<TXCIE)|(0<<UDRIE)|(1<<RXEN)|(1<<TXEN)|(0<<UCSZ2));
	UBRRH=0;
	UBRRL=103;//9600,
	UCSRC=((0<<URSEL)|(0<<UMSEL)|(1<<UPM1)|(0<<UPM0)|(0<<USBS)|(1<<UCSZ1)|(1<<UCSZ0)|(0<<UCPOL));
    
	USS_OUTDDR;
	USS_RXEN;
	UssBusS.Recv.sw = USS_RX_START;


}
