/*
 * Copyright (c) 2009 Michal Sarnovsky
 */


#ifndef __UART0_CONFIG_H__
#define __UART0_CONFIG_H__


/**
 * \addtogroup uart0_config
 *
 * @{
 */
/**
 * \file
 * UART0 configuration
 *
 * \author Michal Sarnovsky
 */


/**
 * UART0 baudrate in [bit/s].
 */
#define UART0_BAUDRATE 9600


/**
 * UART0 receive bytes callback.
 *
 * Set to 0 to saved incoming bytes to RX FIFO buffer, set to 1 to disable RX buffer and return received bytes by callback.
 */
#define UART0_RX_CALLBACK_SUPPORT 0


/**
 * Size of RX FIFO buffer, must be larger then 2.
 *
 * \note This parameter is valid only if RX callback support is disabled. If RX callback support is enabled then even no RX
 *       buffer si allocated.
 */
#define UART0_RX_BUFFER_SIZE 8


/**
 * Size of TX FIFO buffer, must be larger then 2.
 */
#define UART0_TX_BUFFER_SIZE 300


/**
 * Receive data overflow support.
 *
 * Set to 0 to throw overflown RX bytes away, set to 1 to start rewriting the oldest RX data in FIFO queue.
 */
#define UART0_RX_OVERFLOW_REWRITE 0


/**
 * @}
 */

#endif
